import logging

from discord.colour import Color
from discord.commands import SlashCommandGroup
from discord.embeds import Embed
from discord.ext import commands

from aadiscordbot import __branch__, __version__
from aadiscordbot.app_settings import get_site_url

from adieu_discordbot_cogs.helper import unload_cog

logger = logging.getLogger(__name__)


class About(commands.Cog):
    """
    All about me!
    """

    def __init__(self, bot):
        self.bot = bot

    about_commands = SlashCommandGroup("about", "All about the Bot and Auth")

    @about_commands.command(name="discordbot", description="About the Discord Bot")
    async def discordbot(self, ctx):
        """
        All about the bot
        """
        embed = Embed(title="AuthBot: The Authening")
        embed.set_thumbnail(
            url=f"{ctx.bot.user.display_avatar.url}"
        )
        embed.colour = Color.blue()

        embed.description = (
            "This is a multi-de-functional discord bot tailored specifically for Alliance Auth Shenanigans.\n\n"
            "With extra special thanks to all "
            "[Contributors](https://github.com/Solar-Helix-Independent-Transport/allianceauth-discordbot/graphs/contributors)\n"
        )

        embed.set_footer(
            text="Lovingly developed for Init.™ and all EvE Nerds by AaronRin and ArielKable. Customized for Space Force by Jacob and Jernee."
        )

        if not ctx.guild:
            embed.add_field(
                name="Number of Servers:", value=len(self.bot.guilds), inline=True
            )
            members = 0
            for g in self.bot.guilds:
                members += g.member_count
            embed.add_field(name="Unwilling Monitorees:",
                            value=members, inline=True)
            embed.add_field(
                name="Auth Link", value=get_site_url(), inline=False
            )

        embed.add_field(
            name="Version", value=f"{__version__} `{__branch__}`", inline=False
        )

        return await ctx.respond(embed=embed)

    @about_commands.command(name="server", description="About this Discord Server")
    async def server(self, ctx):
        """
        All about a server
        """
        if ctx.guild:
            embed = Embed(title=ctx.guild.name)

            if ctx.guild.icon:
                embed.set_thumbnail(
                    url=ctx.guild.icon.url
                )
            embed.color = Color.blue()
            embed.description = "Alliance Auth Managed EvE Online Discord Server!"
            if ctx.guild.description:
                embed.description = ctx.guild.description
            embed.set_footer(
                text="AuthBot Lovingly developed for Init.™ by AaronRin and ArielKable. Customized for Space Force by Jacob and Jernee.")

            members = ctx.guild.member_count
            embed.add_field(name="Unwilling Monitorees:",
                            value=members, inline=True)

            channels = len(ctx.guild.channels)
            cats = len(ctx.guild.categories)
            embed.add_field(name="Channel Count:",
                            value=channels - cats, inline=True)

            roles = len(ctx.guild.roles)
            embed.add_field(name="Role Count:",
                            value=roles, inline=True)

            embed.add_field(
                name="Auth Link", value=get_site_url(), inline=False
            )

            return await ctx.respond(embed=embed)
        else:
            return await ctx.respond(
                "Sorry, this command cannot be used in DMs."
            )


def setup(bot):
    unload_cog(bot=bot, cog_name="About")
    bot.add_cog(About(bot))


"""
Hooking into the auth system
"""

# Alliance Auth
from allianceauth import hooks


@hooks.register("discord_cogs_hook")
def register_cogs():
    """
    Registering our discord cogs
    :return:
    :rtype:
    """

    return [
        #"adieu_discordbot_cogs.cogs.about",
        #"adieu_discordbot_cogs.cogs.admin",
        #"adieu_discordbot_cogs.cogs.auth",
        "adieu_discordbot_cogs.cogs.recruit_me",
        #"adieu_discordbot_cogs.cogs.welcome",
        "adieu_discordbot_cogs.cogs.migrate",
    ]


# Cog Stuff
import logging

import discord
# AA-Discordbot
from discord.embeds import Embed
from discord.ext import commands

# AA Contexts
from django.conf import settings
from django.utils import timezone
from adieu_discordbot_cogs.helper import unload_cog

from aadiscordbot import app_settings

logger = logging.getLogger(__name__)


class RecruitMe(commands.Cog):
    """
    Thread Tools for recruiting!
    """

    def __init__(self, bot):
        self.bot = bot

    async def open_ticket(
        self,
        ctx: discord.Interaction,
        member: discord.Member
    ):
        sup_channel = settings.RECRUIT_CHANNEL_ID
        ch = ctx.guild.get_channel(sup_channel)
        th = await ch.create_thread(
            name=f"{member.display_name} | {timezone.now().strftime('%Y-%m-%d %H:%M')}",
            auto_archive_duration=10080,
            type=discord.ChannelType.private_thread,
            reason=None
        )
        msg = (f"<@{member.id}> is hunting for a recruiter!\n\n"
               f"Someone from our <@&{settings.RECRUITER_GROUP_ID}> team will get in touch soon!")
        embd = Embed(title="Black Rose Recruitment",
                            description="Welcome! We're glad you're interested in joining us!\n\nIn order to proceed, you will need to set up authentication for all your characters that you will use with our Alliance. Once that's done, we'll finalize the recruitment process with a voice interview with one of our recruiters!\n\nFollow the links below to get started:\n\n> **Sign into our Alliance Authentication service:**\nhttps://auth.black-rose.space/dashboard/\n\n> **After logging in, click on the Login button on this page in the Char Link box:**\nhttps://auth.black-rose.space/charlink/\n\n> **Link your discord account here:**\nhttps://auth.black-rose.space/services/\n\n**Disclaimer**\nThese services **DO NOT** obtain private account information such as username and passwords. We utilize these services to ensure safety within our sovereignty, and provide pilots with information.",
                            colour=0x00b0f4)

        embd.set_thumbnail(url="https://images.evetech.net/alliances/99012770/logo")

        embd.set_footer(text="Crafted for Black Rose, based on AA Discordbot")
        await th.send(msg, embed=embd)
        await ctx.response.send_message(content="Recruitment thread created!", view=None, ephemeral=True)

    @commands.slash_command(
        name='recruit_me',
        guild_ids=app_settings.get_all_servers()
    )
    async def slash_halp(
        self,
        ctx,
    ):
        """
            Get hold of a recruiter
        """
        await self.open_ticket(ctx, ctx.user)

    @commands.message_command(
        name="Create Recruitment Thread",
        guild_ids=app_settings.get_all_servers()
    )
    async def reverse_recruit_msg_context(
        self,
        ctx,
        message
    ):
        """
            Help a new guy get recruiter
        """
        await self.open_ticket(ctx, message.author)

    @commands.user_command(
        name="Recruit Member",
        guild_ids=app_settings.get_all_servers()
    )
    async def reverse_recruit_user_context(
        self, ctx, user
    ):
        """
            Help a new guy get recruiter
        """
        await self.open_ticket(ctx, user)


def setup(bot):
    #unload_cog(bot=bot, cog_name="RecruitMe")
    bot.add_cog(RecruitMe(bot))


"""
Application init
"""

# Third Party
import requests

__version__ = "0.0.8"
__title__ = "ADIEU Discordbot Cogs"

__package_name__ = "adieu-discordbot-cogs"
__package_name_verbose__ = "Black Rose Discordbot Cogs"
__package_name_useragent__ = "ADIEU-Discordbot-Cogs"
__app_name__ = "adieu_discordbot_cogs"
__github_url__ = f"https://github.com/blackrose-eve/{__package_name__}"
__user_agent__ = (
    f"{__package_name_useragent__}/{__version__} "
    f"(+{__github_url__}) requests/{requests.__version__}"
)



# Cog Stuff
import logging

from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

# AA Contexts
from aadiscordbot.app_settings import get_site_url
from adieu_discordbot_cogs.helper import unload_cog

logger = logging.getLogger(__name__)


class Auth(commands.Cog):
    """
    A Collection of Authentication Tools for Alliance Auth
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def auth(self, ctx):
        """
        Returns a link to the AllianceAuth Install
        Used by many other Bots and is a common command that
        users will attempt to run.
        """
        await ctx.trigger_typing()

        embed = Embed(title="AllianceAuth")
        embed.set_thumbnail(
            url="https://assets.gitlab-static.net/uploads/-/system/project/avatar/6840712/Alliance_auth.png?width=128"
        )
        embed.colour = Color.blue()

        embed.description = "All Authentication functions for this Discord server are handled through our Alliance Auth install"

        url = get_site_url()

        embed.add_field(
            name="Auth Link", value=url, inline=False
        )

        return await ctx.send(embed=embed)

    @commands.slash_command(name='auth')
    async def auth_slash(self, ctx):
        """
        Returns a link to the AllianceAuth Install
        Used by many other Bots and is a common command that
        users will attempt to run.
        """
        if ctx.guild:
            embed = Embed(title="AllianceAuth")
            embed.set_thumbnail(
                url="https://assets.gitlab-static.net/uploads/-/system/project/avatar/6840712/Alliance_auth.png?width=128"
            )
            embed.colour = Color.blue()

            embed.description = "All Authentication functions for this Discord server are handled through our Alliance Auth install"

            url = get_site_url()

            embed.add_field(
                name="Auth Link", value=url, inline=False
            )
            embed.set_footer(
                text="Lovingly developed for Init.™ by AaronRin and ArielKable. Customized for Space Force by Jacob and Jernee.")

            return await ctx.respond(embed=embed)

        else:
            return await ctx.respond(
                "Sorry, this command cannot be used in DMs."
            )


def setup(bot):
    unload_cog(bot=bot, cog_name="Auth")
    bot.add_cog(Auth(bot))


# Cog Stuff
import logging

from discord.colour import Color
from discord.embeds import Embed
from discord.ext import commands

# AA Contexts
from aadiscordbot.app_settings import get_site_url
from adieu_discordbot_cogs.helper import unload_cog

logger = logging.getLogger(__name__)


class Migrate(commands.Cog):
    """
    A Collection of Authentication Tools for Alliance Auth
    """

    def __init__(self, bot):
        self.bot = bot

    @commands.command(pass_context=True)
    async def migrate(self, ctx):
        """
        Returns a link to the AllianceAuth Install
        Used by many other Bots and is a common command that
        users will attempt to run.
        """
        await ctx.trigger_typing()

        embd = Embed(title="AllianceAuth")
        embd.set_thumbnail(
            url="https://assets.gitlab-static.net/uploads/-/system/project/avatar/6840712/Alliance_auth.png?width=128"
        )
        embd.colour = Color.blue()

        embd.description = "All Authentication functions for this Discord server are handled through our Alliance Auth install"

        url = get_site_url()

        embd.add_field(
            name="Auth Link", value=url, inline=False
        )

        return await ctx.send(embed=embd)

    @commands.slash_command(name='migrate')
    async def migrate_slash(self, ctx):
        """
        Returns a response about the Migration to Auth from SeAT.
        """
        if ctx.guild:
            embd = Embed(description="We have begun the process of decommissioning SeAT and have begun the migration to Alliance Auth. \n\n**You will need to log into here:**\nhttps://auth.black-rose.space\n\n**Then Activate the Discord Service here:**\nhttps://auth.black-rose.space/services/\n\n**Then apply for groups in here:**\nhttps://auth.black-rose.space/groups/\n\nIf you still need assistance please use:\n```\n/help\n```",
                      colour=0x00b0f4)

            embd.set_author(name="SeAT to Auth Migration!")

            return await ctx.respond(embed=embd)

        else:
            return await ctx.respond(
                "Sorry, this command cannot be used in DMs."
            )


def setup(bot):
    unload_cog(bot=bot, cog_name="Migrate")
    bot.add_cog(Migrate(bot))


"""
Helper functions for the TNNT Discord bot.
"""

# Third Party
from discord.ext import commands


def unload_cog(bot: commands.Bot, cog_name: str) -> None:
    """
    Unloads a cog from the bot.

    :param bot: The bot instance.
    :type bot: discord.ext.commands.Bot
    :param cog_name: The name of the cog to unload.
    :type cog_name: str
    """

    try:
        bot.remove_cog(name=cog_name)

        print(f"Unloaded {cog_name} successfully.")
    except Exception as e:
        print(f"Failed to unload {cog_name}: {e}")